function [Fvec,Xvec] = mkvlqr(Avec,Bvec,Qvec,Lvec,Rvec,P,options,v)

%MKVLQR	Linear quadratic regulater for Markovian systems
%	
%	Usage:
%		[Fvec,Xvec] = mkvlqr(Avec,Bvec,Qvec,Lvec,Rvec,P,OPTIONS,V);
%
%	Fvec contains the controller and Xvec contains the solution to the
%	associated CARE.
%
%	Avec,...,Rvec contain the system and cost description, P is the 
%	transition probability matrix, OPTIONS contains options for the
%	optimization algorithm (type HELP MINCX for more information) and
%	V controls the presentation of information about the problem 
%	construction on the screen (V=1 means verbose mode).

%	Version 1.0
%	RP Marques	(11/03/97)

% Argument checking
if ~( isnan(Avec(1)) & isnan(Bvec(1)) & isnan(Qvec(1)) & isnan(Lvec(1)) & isnan(Rvec(1)) )
	error('Invalid argument(s)')
end; 
N = Avec(2);
if (Bvec(2)~=N) | (Qvec(2)~=N) | (Lvec(2)~=N) | (Rvec(2)~=N)
	error('Invalid number of operation modes for some argument(s)')
end;

if nargin<8, v=0; end;
if nargin<7, options=zeros(1,5); end;

if v==[], v=0; end;
if options==[], options=zeros(1,5); end;

% Optimization
Xvec = maxcare(Avec,Bvec,Qvec,Lvec,Rvec,P,options,v);

% Controller construction
if v, fprintf('\nController construction.\n\n'); end;
EXvec = eop(Xvec,P);
str=[];
for i=1:N,
	istr=int2str(i); aux=[',F',istr]; str=[str,aux];
	Ai=unstack(Avec,i); Bi=unstack(Bvec,i);
	Ri=unstack(Rvec,i); Li=unstack(Lvec,i);
	Ei=unstack(EXvec,i);
	Fi=-inv(Bi'*Ei*Bi+Ri)*(Bi'*Ei*Ai+Li');
	eval(['F',istr,'=Fi;']);
end;
str(1)=[]; eval(['Fvec=stack(',str,');']);

% Stability checking
if v, fprintf('\nStability checking.\n\n'); end;
Gvec = cloopvec(Avec,Bvec,Fvec);
if specrad(Gvec,P)>=1
	error('Stability checking failed. Obtained solution is not stabilizing');
end;

